/*-*- c++ -*-********************************************************
 * wxMainFrame: the main window class                               *
 *                                                                  *
 * (C) 1998-2000 by Karsten Ballder (Ballueder@gmx.net)            *
 *                                                                  *
 * $Id: wxMainFrame.cpp,v 1.107 2000/10/02 18:46:59 vadz Exp $
 *
 *******************************************************************/

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

#ifdef __GNUG__
#   pragma   implementation "wxMainFrame.h"
#endif

#include   "Mpch.h"

#ifndef   USE_PCH
#   include   "Mcommon.h"
#   include   "guidef.h"
#   include   "MFrame.h"
#   include   "Mdefaults.h"
#   include   "strutil.h"
#endif

#include "wx/persctrl.h"
#include <wx/menuitem.h>

#include "MFolder.h"

#include "gui/wxMainFrame.h"
#include "gui/wxMApp.h"
#include "gui/wxIconManager.h"
#include "gui/wxFolderView.h"
#include "gui/wxFolderTree.h"

#include "gui/wxFiltersDialog.h" // for ConfigureFiltersForFolder
#include "gui/wxIdentityCombo.h" // for IDC_IDENT_COMBO

#include "MFolderDialogs.h"      // for ShowFolderPropertiesDialog
#include "miscutil.h"            // for UpdateTitleAndStatusBars

#include "MHelp.h"
#include "MDialogs.h"

#include "MModule.h"

// define this for future, less broken versions of wxWindows to dynamically
// insert/remove the message menu depending on whether we have or not a folder
// view in the frame - with current wxGTK it doesn't work at all, so disabling
#undef HAS_DYNAMIC_MENU_SUPPORT

// ----------------------------------------------------------------------------
// private classes
// ----------------------------------------------------------------------------

#if 0
// A popup menu for entering and configuring plugin modules
class wxModulePopup : public wxMenu
{
public:
   static wxModulePopup * Create(void);
   // callbacks
   void OnCommandEvent(wxCommandEvent &event);
   ~wxModulePopup()
      { SafeDecRef(m_Listing); }

private:
   wxModulePopup();
   MModuleListing *m_Listing;
   size_t m_CountModules, m_CountMain, m_CountConfig;
   DECLARE_EVENT_TABLE()
};


/* static */
wxModulePopup *
wxModulePopup::Create(void)
{
   wxModulePopup * popup = new wxModulePopup();
   if(popup->m_CountMain + popup->m_CountConfig == 0) // nothing to do
   {
      delete popup;
      return NULL;
   }
   return popup;
}


wxModulePopup::wxModulePopup()
   : wxMenu(_("Plugin Modules"))
{
   m_CountMain = 0;
   m_CountConfig = 0;

   m_Listing = MModule::ListLoadedModules();
   m_CountModules = m_Listing->Count();
   for(size_t i = 0; i < m_CountModules; i++)
   {
      MModule *mod = (*m_Listing)[i].GetModule();
      if(mod->Entry(MMOD_FUNC_GETFLAGS) & MMOD_FLAG_HASMAIN)
         m_CountMain++;
      if(mod->Entry(MMOD_FUNC_GETFLAGS) & MMOD_FLAG_HASCONFIG)
         m_CountConfig++;
      mod->DecRef();
   }

   if(m_CountMain+m_CountConfig == 0)
   {
      /// dummy menu:
      Append(WXMENU_POPUP_MODULES_OFFS,_("No modules to run or configure!"));
      Enable(WXMENU_POPUP_MODULES_OFFS, FALSE);
   }
   else
   {
      if(m_CountMain)
      {
         for(size_t i = 0; i < m_Listing->Count(); i++)
         {
            wxString entry;
            MModule *mod = (*m_Listing)[i].GetModule();
            if(mod->Entry(MMOD_FUNC_GETFLAGS) & MMOD_FLAG_HASMAIN)
            {
               entry = (*m_Listing)[i].GetName();
               entry << _(" module...");
               Append(WXMENU_POPUP_MODULES_OFFS+i, entry);
            }
            mod->DecRef();
         }
      }
      if(m_CountMain && m_CountConfig)
         AppendSeparator();
      if(m_CountConfig)
      {
         for(size_t i = 0; i < m_Listing->Count(); i++)
         {
            wxString entry;
            MModule *mod = (*m_Listing)[i].GetModule();
            if(mod->Entry(MMOD_FUNC_GETFLAGS) & MMOD_FLAG_HASCONFIG)
            {
               entry.Printf(_("Configure %s module..."),
                            (*m_Listing)[i].GetName().c_str());
               Append(WXMENU_POPUP_MODULES_OFFS+i+m_CountModules, entry);
            }
            mod->DecRef();
         }
      }
   }
}

void
wxModulePopup::OnCommandEvent(wxCommandEvent &event)
{
   size_t id = (size_t) event.GetId();

   if(id < 0) // wxGTK menu title (how stupid!)
      return;

   id = id-WXMENU_POPUP_MODULES_OFFS;
   if(id < m_CountModules)
   {
      MModule *mod = (*m_Listing)[id].GetModule();
      mod->Entry(MMOD_FUNC_MAIN);
      mod->DecRef();
      return;
   }
   else
   {
      id -= m_CountModules;
      MModule *mod = (*m_Listing)[id].GetModule();
      mod->Entry(MMOD_FUNC_CONFIG);
      mod->DecRef();
      return;
   }
   ASSERT(0 /* unreachable code */);
}
BEGIN_EVENT_TABLE(wxModulePopup, wxMenu)
  EVT_MENU(-1,    wxModulePopup::OnCommandEvent)
END_EVENT_TABLE()

#endif // 0

// override wxFolderTree OnOpenHere() function to open the folder in this
// frame and OnClose() to close it
class wxMainFolderTree : public wxFolderTree
{
public:
   wxMainFolderTree(wxWindow *parent, wxMainFrame *frame)
      : wxFolderTree(parent)
   {
      m_frame = frame;
   }

   virtual void OnSelectionChange(MFolder *oldsel, MFolder *newsel)
   {
      if ( newsel )
      {
         wxLogStatus(m_frame, _("Selected folder '%s'."),
                     newsel->GetFullName().c_str());

         UpdateMenu(m_frame->GetMenuBar()->GetMenu(MMenu_Folder), newsel);
      }

      wxFolderTree::OnSelectionChange(oldsel, newsel);
   }

   virtual void OnOpenHere(MFolder *folder)
   {
      // attention, base version of OnOpenHere() will DecRef() the folder, so
      // compensate for it
      SafeIncRef(folder);
      wxFolderTree::OnOpenHere(folder);
      m_frame->OpenFolder(folder);
   }

   virtual bool OnClose(MFolder *folder)
   {
      m_frame->CloseFolder(folder);

      return TRUE;
   }

private:
   wxMainFrame *m_frame;
};

// ----------------------------------------------------------------------------
// event tables
// ----------------------------------------------------------------------------

BEGIN_EVENT_TABLE(wxMainFrame, wxMFrame)
   EVT_MENU(-1,    wxMainFrame::OnCommandEvent)
   EVT_TOOL(-1,    wxMainFrame::OnCommandEvent)

   EVT_CHOICE(IDC_IDENT_COMBO, wxMainFrame::OnIdentChange)
END_EVENT_TABLE()

// ============================================================================
// implementation
// ============================================================================

wxMainFrame::wxMainFrame(const String &iname, wxFrame *parent)
           : wxMFrame(iname,parent)
{
   SetIcon(ICON("MainFrame"));
   SetTitle(M_TOPLEVELFRAME_TITLE);

   static int widths[3] = { -1, 70, 100 }; // FIXME: temporary for debugging
   CreateStatusBar(3, wxST_SIZEGRIP, 12345); // 3 fields, id 12345 fo
   GetStatusBar()->SetFieldsCount(3, widths);

   // construct the menu and toolbar
   AddFileMenu();
   AddFolderMenu();
   AddEditMenu();

#ifndef HAS_DYNAMIC_MENU_SUPPORT
   AddMessageMenu();
   AddLanguageMenu();
#endif

   AddHelpMenu();
   AddToolbarButtons(CreateToolBar(), WXFRAME_MAIN);

   // disable the operations which don't make sense for viewer
   wxMenuBar *menuBar = GetMenuBar();
   menuBar->Enable(WXMENU_EDIT_CUT, FALSE);
   menuBar->Enable(WXMENU_EDIT_PASTE, FALSE);

   m_ModulesMenu = NULL;

   int x,y;
   GetClientSize(&x, &y);

   m_splitter = new wxPSplitterWindow
                    (
                     "MainSplitter",
                     this,
                     -1,
                     wxPoint(1,31),       // FIXME what is this "31"?
                     wxSize(x-1,y-31),
                     wxSP_3D | wxSP_BORDER
                    );
   // insert treectrl in one of the splitter panes
   m_FolderTree = new wxMainFolderTree(m_splitter, this);
   m_FolderView = wxFolderView::Create(m_splitter);
   m_splitter->SplitVertically(m_FolderTree->GetWindow(),
                               m_FolderView->GetWindow(),
                               x/3);

   // open the last folder in the main frame by default
   String foldername = READ_APPCONFIG(MP_MAINFOLDER);
   if ( !foldername.IsEmpty() )
   {
      MFolder *folder = MFolder::Get(foldername);
      if ( folder )
      {
         // make sure it doesn't go away after OpenFolder()
         folder->IncRef();
         OpenFolder(folder);
         m_FolderTree->SelectFolder(folder);
         folder->DecRef();
      }
   }

   m_splitter->SetMinimumPaneSize(0);
   m_splitter->SetFocus();

   //FIXME: ugly workaround for wxGTK toolbar/resize bug
#ifdef __WXGTK__
   {
     int x,y;
     GetSize(&x,&y);
     SetSize(x,y+1);
     SetSize(x,y);
   }
#endif // GTK
}

void wxMainFrame::AddFolderMenu(void)
{
   WXADD_MENU(GetMenuBar(), FOLDER, _("&Folder"));
}

void
wxMainFrame::CloseFolder(MFolder *folder)
{
   if ( folder && folder->GetFullName() == m_folderName )
   {
      m_FolderView->SetFolder(NULL);

      m_folderName.clear();
   }
   //else: otherwise, we don't have it opened
}

void
wxMainFrame::OpenFolder(MFolder *pFolder)
{
#ifdef HAS_DYNAMIC_MENU_SUPPORT
   static bool s_hasMsgMenu = false;
#endif

   MFolder_obj folder(pFolder);

   // don't do anything if there is nothing to change
   if ( folder.IsOk() && (m_folderName == folder->GetFullName()) )
   {
      // ... unless opening the folder previously failed, in which case we'll
      // try to reopen it (may be the user changed some of its settings)
      bool reopen = FALSE;

      int flags = folder->GetFlags();
      if ( flags & MF_FLAGS_MODIFIED )
      {
         // user changed some settings, try opening the folder again without
         // asking all sorts of dumb questions (like below)
         reopen = TRUE;
      }
      else if ( flags & MF_FLAGS_UNACCESSIBLE )
      {
         if ( MDialog_YesNoDialog(_("This folder couldn't be opened last time, "
                                    "do you still want to try to open it (it "
                                    "will probably fail again)?"),
                                  this,
                                  MDIALOG_YESNOTITLE,
                                  FALSE,
                                  "OpenUnaccessibleFolder") )
         {
            if ( MDialog_YesNoDialog(_("Would you like to change folder "
                                       "settings before trying to open it?"),
                                     this,
                                     MDIALOG_YESNOTITLE,
                                     FALSE,
                                     "ChangeUnaccessibleFolderSettings") )
            {
               // invoke the folder properties dialog
               (void)ShowFolderPropertiesDialog(folder, this);
            }

            reopen = TRUE;
         }
      }

      if ( !reopen )
         return;
   }
   else if ( folder )
      m_folderName = folder->GetFullName();
   else if ( m_folderName.IsEmpty() )
      return;
   else
      m_folderName.Empty();

   if ( folder )
   {
      // we want save the full folder name in m_folderName
      ASSERT( folder->GetFullName() == m_folderName );

      MailFolder *mailFolder = m_FolderView->OpenFolder(folder->GetFullName());
      if ( mailFolder )
      {
         mailFolder->IncRef();
         // reset the unaccessible and modified flags this folder might have
         // had
         folder->ResetFlags(MF_FLAGS_MODIFIED | MF_FLAGS_UNACCESSIBLE);

         UpdateTitleAndStatusBars("", _("Opened folder: "), this, mailFolder);
         mailFolder->DecRef();
      }
      else if ( mApplication->GetLastError() == M_ERROR_CANCEL )
      {
         // don't set the unaccessible flag - may be it's ok
         wxLogStatus(this, _("Opening folder '%s' cancelled."),
                     m_folderName.c_str());
      }
      else
      {
         // it's not modified any more...
         folder->ResetFlags(MF_FLAGS_MODIFIED);

         // ... and it is unacessible because we couldn't open it
         folder->AddFlags(MF_FLAGS_UNACCESSIBLE);

         m_folderName.clear();
      }
  }
#ifdef HAS_DYNAMIC_MENU_SUPPORT
   // only add the msg menu once
   if ( !s_hasMsgMenu )
   {
      AddMessageMenu();
      s_hasMsgMenu = true;
   }
#endif // HAS_DYNAMIC_MENU_SUPPORT

#ifdef HAS_DYNAMIC_MENU_SUPPORT
      // TODO remove the message menu - wxMenuBar::Delete() not implemented
      //      currently in wxGTK and is somewhat broken in wxMSW
#endif // HAS_DYNAMIC_MENU_SUPPORT
}

bool
wxMainFrame::CanClose() const
{
   // closing the main frame will close the app so ask the other frames
   // whether it's ok to close them
   bool rc = mApplication->CanClose();
   // make sure folder is closed before we close the window
   if(rc == TRUE)
      m_FolderView->SetFolder(NULL);
   return rc;
}

void
wxMainFrame::OnIdentChange(wxCommandEvent &event)
{
   Profile *profile = mApplication->GetProfile();
   if ( event.GetInt() == 0 )
   {
      // restore the default identity
      profile->DeleteEntry(MP_CURRENT_IDENTITY);
   }
   else
   {
      profile->writeEntry(MP_CURRENT_IDENTITY, event.GetString());
   }

   // TODO: update everything on identity change
}

void
wxMainFrame::OnCommandEvent(wxCommandEvent &event)
{
   int id = event.GetId();

   if ( WXMENU_CONTAINS(FOLDER, id) )
   {
      switch ( id )
      {
         case WXMENU_FOLDER_OPEN:
            {
               wxString name;
               if ( MInputBox(&name, _("Folder Open"), _("Name of the folder?"),
                              this, "OpenFolderName", "INBOX") )
               {
                  (void) wxFolderViewFrame::Create(name, this);
               }
            }
            break;

         case WXMENU_FOLDER_CREATE:
            {
               wxWindow *winTop = ((wxMApp *)mApplication)->GetTopWindow();
               bool wantsDialog;
               MFolder *newfolder = RunCreateFolderWizard(&wantsDialog, NULL, winTop);
               if ( wantsDialog )
               {
                  // users wants to use the dialog directly instead of the wizard
                  newfolder = ShowFolderCreateDialog(winTop, FolderCreatePage_Default, NULL);
               }
               if(newfolder)
                  newfolder->DecRef();
            }
            break;

         case WXMENU_FOLDER_FILTERS:
            {
               MFolder_obj folder(m_folderName);
               if ( folder )
                  ConfigureFiltersForFolder(folder, this);
            }
            break;

         default:
            // all others are processed by the folder tree
            m_FolderTree->ProcessMenuCommand(id);
      }
   }
   else if ( m_FolderView &&
            (WXMENU_CONTAINS(MSG, id) ||
             WXMENU_CONTAINS(LAYOUT, id) ||
             (WXMENU_CONTAINS(LANG, id) && (id != WXMENU_LANG_SET_DEFAULT)) ||
             id == WXMENU_FILE_COMPOSE ||
             id == WXMENU_FILE_POST ||
             id == WXMENU_EDIT_COPY ) )
      m_FolderView->OnCommandEvent(event);
   else if(id == WXMENU_HELP_CONTEXT)
      mApplication->Help(MH_MAIN_FRAME,this);
   else
   {
      wxMFrame::OnMenuCommand(id);
   }
}

wxMainFrame::~wxMainFrame()
{
   // tell the app there is no main frame any more and do it as the very first
   // thing because the next calls may result in message boxes being displayed
   // and they shouldn't have this frame as a parent because it can go away at
   // any moment
   mApplication->OnMainFrameClose();

   delete m_FolderView;
   delete m_FolderTree;
   // save the last opened folder
   mApplication->GetProfile()->writeEntry(MP_MAINFOLDER, m_folderName);
}

void
wxMainFrame::MakeModulesMenu(void)
{
   wxMenuBar *menuBar = GetMenuBar();
   if(! m_ModulesMenu)
   {
      // create the modules menu:
      m_ModulesMenu = new wxMenu("", wxMENU_TEAROFF);
      int pos = menuBar->GetMenuCount();
      wxASSERT(pos  > 1);
      // pos is the Help menu, so insert at pos-1:
      menuBar->Insert(pos-1, m_ModulesMenu, _("&Plugins"));
   }
}

/// Appends the menu for a module to the menubar
void
wxMainFrame::AddModulesMenu(const char *name,
                            const char *help,
                            class wxMenu *submenu,
                            int id)
{
   wxASSERT(submenu);
   MakeModulesMenu();
   if(id == -1)
      id = NewControlId();
   m_ModulesMenu->Append(id, name, submenu, help);
}


/// Appends the menu entry for a module to the modules menu
void
wxMainFrame::AddModulesMenu(const char *name,
                            const char *help,
                            int id)
{
   MakeModulesMenu();
   m_ModulesMenu->Append(id, name, help);
}

Profile *
wxMainFrame::GetFolderProfile(void)
{
   return m_FolderView ? m_FolderView->GetProfile() : NULL;
}

